#include <stdlib.h> /* calloc */
#include <stdio.h>  /* printf */
#include <assert.h>
#include <sys/time.h>
#include <string.h> /* memcpy */
#include "../../common/common.h"


/* Within bounds or die */
#define inBounds(i,j) assert(0<=(i) && (i)<N && 0<=(j) && (j)<N)

#define KERNEL_FLOP 4
#define KERNEL_MOP 5

/* Kernel */
inline void heat(float *old, float *new)
{
	unsigned int i = 0, j = 0;
	assert(old && new);
	
	for (i=1; i<N-1; i++) {
		for (j=1; j<N-1; j++) {
			inBounds(i-1,j); inBounds(i,j-1);
			inBounds(i,j+1); inBounds(i+1,j);
			new[INDEX(i,j)] = 0.25f*(
					old[INDEX(i-1,j)]+
				old[INDEX(i,j-1)] + old[INDEX(i,j+1)] +
					old[INDEX(i+1,j)]
						);
		}
	}
}


int main(int argc, char** argv)
{
	unsigned int i = 0;
	/* At least a border square surrounding a single cell */
	assert(2<=N);

	/* Allocate matrix as vector */
	size_t size = N*N * sizeof(float);
	assert(0<size);

	float *h_old = NULL;
	float *h_new = NULL;

	/* Alloc and set memory */
	h_old = (float *)calloc(1, size);
	assert(h_old);
	h_new = (float *)calloc(1, size);
	assert(h_old);
	/* Surroundings on fire */
	for (i=0; i<N; i++) {
		float value = HOT - (4.0/(N*N) * HOT)*(N/2.0-i)*(N/2.0-i);
		h_old[INDEX(0,i)] = h_old[INDEX(N-1,i)] = value;
		h_old[INDEX(i,0)] = h_old[INDEX(i,N-1)] = value;
	}
	memcpy(h_new, h_old, size);
	/* There is no temp one pixel off the center */
	assert(0.0==h_old[OFFBYONE_CORNER]);
	/* Before */
	writePPMbinaryImage("image_before.ppm", h_new);

	/* Start timer */
	double secs = 0.0;
	struct timeval start = {0L,0L}, end = {0L,0L}, elapsed = {0L,0L};
	gettimeofday(&start,NULL);

	/* Invoke kernel */
	for(i=0; i<MAX_ITERATIONS; i++) {
		float *swap = NULL;
		heat(h_old, h_new);
		swap = h_old; h_old = h_new; h_new = swap;
	}

	/* Stop timer */
	gettimeofday(&end,NULL);
	timeval_subtract(&elapsed, &end, &start);
	secs = (double)elapsed.tv_sec + ((double)elapsed.tv_usec*MICROSEC);

	/* Check for correctness, diagonal neighbors have some heat */
	assert(0.0<h_new[OFFBYONE_CORNER]);

	/* After */
	writePPMbinaryImage("image_after.ppm", h_new);
	
	/* Print stats */
	printf("Secs: %f\n", secs);
	printf("GBps: %f\n", ((float)N*N*sizeof(float)*KERNEL_MOP*MAX_ITERATIONS / secs)*1e-9);
	printf("GFlops: %f\n", ((float)N*N*KERNEL_FLOP*MAX_ITERATIONS / secs)*1e-9);

	/* Free device/host memory */
	free(h_old); h_old = NULL;
	free(h_new); h_new = NULL;

	return 0;
}
