#include <stdlib.h> /* malloc */
#include <stdio.h>  /* printf */
#include <assert.h>
#include <sys/time.h>
#include "../../common/common.h"


int main(int argc, char** argv)
{
	unsigned int M = 0;
	unsigned int i = 0;

	/* Handle input params */
	if (argc!=2) {
		printf("Usage: %s 2^<bits>\n", argv[0]);
		exit(1);
	}
	assert(argc==2);
	for (i=0; i<argc; i++) {
		assert(argv[i]);
	}
	M = 1 << atoi(argv[1]);

	/* Allocate vectors in device memory */
	size_t size = M * sizeof(float);
	assert(0<size);

	float a = 2.0;
	float *h_x = NULL, *h_y = NULL;

	/* Alloc and set host memory */
	h_x = (float *)malloc(size);
	assert(h_x);	
	h_y= (float *)malloc(size);
	assert(h_y);	
	for (i=0; i<M; i++) {
		h_x[i] = h_y[i] = 2.0;
	}

	/* Start timer */
	double secs = 0.0;
	struct timeval start = {0L,0L}, end = {0L,0L}, elapsed = {0L,0L};
	gettimeofday(&start, NULL);

	/* Invoke kernel */
#define KERNEL_MOP 3
#define KERNEL_FLOP 2
	for (i=0; i<M; i++) {
		h_y[i] += a*h_x[i];
	}

	/* Stop timer */
	gettimeofday(&end, NULL);
	timeval_subtract(&elapsed, &end, &start);
	secs = (double)elapsed.tv_sec + ((double)elapsed.tv_usec*MICROSEC);

	/* Check for correctness */
	for (i=0; i<M; i++) {
		assert(h_y[i]==2*h_x[i]+h_x[i]);
	}

	/* Print results */
	printf("Secs: %f\n", secs);
	printf("GBps: %f\n", ((float)M*sizeof(float)*KERNEL_MOP / secs)*1e-9);
	printf("GFlops: %f\n", (M*KERNEL_FLOP / secs)*1e-9);	

	/* Free host memory */
	free(h_x); h_x = NULL;
	free(h_y); h_y = NULL;

	return 0;
}
