#include <stdio.h> /* FILE * ops */
#include <assert.h>
#include <sys/time.h> /* struct timeval */
#include <stdint.h> /* uint8_t */
#include "common.h"


/* Write to a PPM image
 * http://en.wikipedia.org/wiki/Portable_Gray_Map
 * http://netpbm.sourceforge.net/doc/ppm.html
 */
 
#define MAX_COMPONENT_VALUE 255
void writePPMbinaryImage(char *filename, float *vector)
{
	FILE *f = NULL;
	unsigned int i = 0, j = 0;
	uint8_t RGB[3] = {0,0,0}; /* use 3 first bytes for BGR */ 
	f = fopen(filename, "w");
	assert(f);
	fprintf(f, "P6\n"); /* Portable colormap, binary */
	fprintf(f, "%d %d\n", N, N); /* Image size */
	fprintf(f, "%d\n", MAX_COMPONENT_VALUE); /* Max component value */
	for (i=0; i<N; i++) {
		for (j=0; j<N; j++) {
			/* ColorMaps 
			 * http://mainline.brynmawr.edu/Courses/cs120/spring2008/Labs/ColorMaps/colorMap.py
			 */
			float weight = (vector[INDEX(i,j)]/HOT); /* [0,1] */
			RGB[0] = (uint8_t) (255*weight+0);
			RGB[1] = (uint8_t) (0);
			RGB[2] = (uint8_t) (-255*weight+255);
			fwrite((void *)RGB, 3, 1, f);
		}
	}
	fclose(f);
	return;
}


/*
 * http://www.gnu.org/software/libtool/manual/libc/Elapsed-Time.html
 * Subtract the `struct timeval' values X and Y,
 * storing the result in RESULT.
 * Return 1 if the difference is negative, otherwise 0.
 */

int timeval_subtract (struct timeval *result, struct timeval *x, struct timeval *y) {
	/* Perform the carry for the later subtraction by updating y. */
	if (x->tv_usec < y->tv_usec) {
		int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
		y->tv_usec -= 1000000 * nsec;
		y->tv_sec += nsec;
	}
	if (x->tv_usec - y->tv_usec > 1000000) {
		int nsec = (x->tv_usec - y->tv_usec) / 1000000;
		y->tv_usec += 1000000 * nsec;
		y->tv_sec -= nsec;
	}

	/* Compute the time remaining to wait. tv_usec is certainly positive. */
	result->tv_sec = x->tv_sec - y->tv_sec;
	result->tv_usec = x->tv_usec - y->tv_usec;

	/* Return 1 if result is negative. */
	return x->tv_sec < y->tv_sec;
}
