#ifndef COMMONCUDA_H
#define COMMONCUDA_H

#include <sys/time.h> /* struct timeval */


/* Integer ceiling division */
#define DIV_CEIL(a,b) (((a)+(b)-1)/(b))
/* Maximum */
#define MAX(a,b) ((a < b) ?  (b) : (a))
/* Vector as a matrix */
#define INDEX(i,j) (((i)*N)+(j))
/* Cell in the right of center */
#define OFFBYONE_CORNER (INDEX(1,1))
/* A time constant */
#define MICROSEC (1E-6)


/* Problem size */
#ifndef N /* default: 1024^2 cells */
#define N (1<<10)
#endif

/* Tile size */
#ifndef TILE /* default: 16^2 cells */
#define TILE 16
#endif

/* or ... Maximum iterations */
#ifndef MAX_ITERATIONS /* default: 1000 times */
#define MAX_ITERATIONS 1000000
#endif

/* Maximum difference */
#ifndef EPSILON /* default: 0.01f */
#define EPSILON (0.02f)
#endif

/* Hot temperature */
#ifndef HOT /* default: 1000.0f degees */
#define HOT (100.0f)
#endif

/* 0 is for Dionisio, 1 is for Eze */
#ifndef DEVICE /* default: 0 */
#define DEVICE 0
#endif


/* Write to a PPM image */
void writePPMbinaryImage(char *filename, float *vector);

/*
 * http://www.gnu.org/software/libtool/manual/libc/Elapsed-Time.html
 * Subtract the `struct timeval' values X and Y,
 * storing the result in RESULT.
 * Return 1 if the difference is negative, otherwise 0.
 */
int timeval_subtract (struct timeval *result, struct timeval *x, struct timeval *y);

#endif /* COMMONCUDA_H */
